==> matrix_sdk-0.8.2.tar.gz: lib/src/event/timeline.dart <==
// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/storables.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:meta/meta.dart';

/// Events map that will lazily load from the given
/// [store].
class Timeline extends Storables<EventId, RoomEvent> {

  final Room _room;

  Timeline(this._room, {Store store}) : super(store: store);

  @override
  @protected
  Future<void> saveAll(Iterable<RoomEvent> events)
    => store?.addRoomEvents(_room, events);

  @override
  @protected
  Future<RoomEvent> retrieve(EventId id) => store?.getRoomEvent(_room, id);

  @override
  @protected
  Stream<RoomEvent> retrieveAll() => store?.getRoomEvents(_room);

  @override
  @protected
  Stream<RoomEvent> retrieveUpTo(int count, {@required Event from})
    => store?.getRoomEvents(_room, count: count, from: from as RoomEvent);

  @protected
  @override
  Stream<RoomEvent> retrieveRemote(int count, {@required Event from})
    => _room.retrieveRemoteEvents(count);

  @protected
  @override
  Future<void> removeInStore(EventId id) async {
    await store?.removeEvent(id);
  }

  @override
  Future<void> addAll(Iterable<RoomEvent> values) async {

    // Remove echo events
    for (final event in values) {
      final transactionId = event?.unsigned?.transactionId;
      if (transactionId != null) {
        // delete local echo
        await remove(EventId(transactionId));
      }
    }

    await super.addAll(values);
  }

  @override
  Iterable<RoomEvent> sort(Iterable<Event> items) {
    final list = items.toList(growable: false);
    list.sort((a, b) {
      if (a is RoomEvent && b is RoomEvent) {
        return a.time.compareTo(b.time);
      } else {
        return a.id.toString().compareTo(b.id.toString());
      }
    });

    return list.reversed;
  }
}

==> matrix_sdk-0.11.0.tar.gz: lib/src/event/timeline.dart <==
// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/storables.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:meta/meta.dart';

/// Events map that will lazily load from the given
/// [store].
class Timeline extends Storables<EventId, RoomEvent> {

  final Room _room;

  Timeline(this._room, {Store store}) : super(store: store);

  @override
  @protected
  Future<void> saveAll(Iterable<RoomEvent> events)
    => store?.addRoomEvents(_room, events);

  @override
  @protected
  Future<RoomEvent> retrieve(EventId id) => store?.getRoomEvent(_room, id);

  @override
  @protected
  Stream<RoomEvent> retrieveAll() => store?.getRoomEvents(_room);

  @override
  @protected
  Stream<RoomEvent> retrieveUpTo({int count, Event from, Event until})
    => store?.getRoomEvents(_room, count: count, from: from as RoomEvent, until: until as RoomEvent);

  @protected
  @override
  Stream<RoomEvent> retrieveRemote(int count, {Event from})
    => _room.retrieveRemoteEvents(count);

  @protected
  @override
  Future<void> removeInStore(EventId id) async {
    await store?.removeEvent(id);
  }

  @override
  Future<void> addAll(Iterable<RoomEvent> values) async {

    // Remove echo events
    for (final event in values) {
      final transactionId = event?.unsigned?.transactionId;
      if (transactionId != null) {
        // delete local echo
        await remove(EventId(transactionId));
      }
    }

    await super.addAll(values);
  }

  @override
  Iterable<RoomEvent> sort(Iterable<Event> items) {
    final list = items.toList(growable: false);
    list.sort((a, b) {
      if (a is RoomEvent && b is RoomEvent) {
        return a.time.compareTo(b.time);
      } else {
        return a.id.toString().compareTo(b.id.toString());
      }
    });

    return list.reversed;
  }
}
